@@ -91,6 +91,7 @@ extern pid_t lookup_initpid_in_store(pid_t qpid);
 extern void prune_init_slice(char *cg);
 extern bool supports_pidfd(void);
 extern bool liblxcfs_functional(void);
+extern bool liblxcfs_can_use_swap(void);
 
 static inline int install_signal_handler(int signo,
 					 void (*handler)(int, siginfo_t *, void *))
@@ -103,6 +104,20 @@ static inline int install_signal_handler(int signo,
 	return sigaction(signo, &action, NULL);
 }
 
-extern pid_t lxcfs_clone(int (*fn)(void *), void *arg, int flags);
+extern pid_t lxcfs_raw_clone(unsigned long flags, int *pidfd);
+
+static inline pid_t lxcfs_clone(int (*fn)(void *), void *arg, int flags)
+{
+	pid_t pid;
+
+	pid = lxcfs_raw_clone(flags, NULL);
+	if (pid < 0)
+		return -1;
+
+	if (pid == 0)
+		_exit(fn(arg));
+
+	return pid;
+}
 
 #endif /* __LXCFS_BINDINGS_H */
