@@ -16,9 +16,17 @@
 #define _GNU_SOURCE
 #endif
 
+#include "../config.h"
+
+#ifdef HAVE_FUSE3
+#ifndef FUSE_USE_VERSION
+#define FUSE_USE_VERSION 30
+#endif
+#else
 #ifndef FUSE_USE_VERSION
 #define FUSE_USE_VERSION 26
 #endif
+#endif
 
 #define _FILE_OFFSET_BITS 64
 
@@ -38,7 +46,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "../config.h"
 #include "../macro.h"
 #include "../memory_utils.h"
 #include "../utils.h"
@@ -613,6 +620,12 @@ static int cgfsng_get_memory_max(struct cgroup_ops *ops, const char *cgroup,
 	return cgfsng_get_memory(ops, cgroup, "memory.max", value);
 }
 
+static int cgfsng_get_memory_swappiness(struct cgroup_ops *ops, const char *cgroup,
+				 char **value)
+{
+	return cgfsng_get_memory(ops, cgroup, "memory.swappiness", value);
+}
+
 static int cgfsng_get_memory_swap_max(struct cgroup_ops *ops,
 				      const char *cgroup, char **value)
 {
@@ -1004,6 +1017,7 @@ struct cgroup_ops *cgfsng_ops_init(void)
 	cgfsng_ops->get_memory_stats_fd = cgfsng_get_memory_stats_fd;
 	cgfsng_ops->get_memory_stats = cgfsng_get_memory_stats;
 	cgfsng_ops->get_memory_max = cgfsng_get_memory_max;
+	cgfsng_ops->get_memory_swappiness = cgfsng_get_memory_swappiness;
 	cgfsng_ops->get_memory_swap_max = cgfsng_get_memory_swap_max;
 	cgfsng_ops->get_memory_current = cgfsng_get_memory_current;
 	cgfsng_ops->get_memory_swap_current = cgfsng_get_memory_swap_current;
