@@ -1,23 +1,27 @@
-#!/bin/bash
+#!/bin/sh
+# SPDX-License-Identifier: LGPL-2.1+
 
-set -ex
+set -eu
+[ -n "${DEBUG:-}" ] && set -x
 
 [ $(id -u) -eq 0 ]
 
 # Run lxcfs testsuite
 export LXCFSDIR=$(mktemp -d)
 pidfile=$(mktemp)
+export LXCFSPID=-1
 
 cmdline=$(realpath $0)
 dirname=$(dirname ${cmdline})
 topdir=$(dirname ${dirname})
 
-p=-1
 FAILED=1
+UNSHARE=1
 cleanup() {
+	echo "=> Cleaning up"
 	set +e
-	if [ $p -ne -1 ]; then
-		kill -9 $p
+	if [ $LXCFSPID -ne -1 ]; then
+		kill -9 $LXCFSPID
 	fi
 	if [ ${LXCFSDIR} != "/var/lib/lxcfs" ]; then
 		umount -l ${LXCFSDIR}
@@ -25,28 +29,34 @@ cleanup() {
 	fi
 	rm -f ${pidfile}
 	if [ ${FAILED} -eq 1 ]; then
-		echo "FAILED at $TESTCASE"
+		echo "=> FAILED at $TESTCASE"
 		exit 1
 	fi
-	echo PASSED
+	echo "=> PASSED"
 	exit 0
 }
 
 TESTCASE="setup"
-lxcfs=${topdir}/lxcfs
+lxcfs=${topdir}/src/lxcfs
 
 if [ -x ${lxcfs} ]; then
-	echo "Running ${lxcfs} ${LXCFSDIR}"
+	if [ -n "${LD_LIBRARY_PATH:-}" ]; then
+		export LD_LIBRARY_PATH="${topdir}/src/.libs/:${LD_LIBRARY_PATH}"
+	else
+		export LD_LIBRARY_PATH="${topdir}/src/.libs/"
+	fi
+	echo "=> Spawning ${lxcfs} ${LXCFSDIR}"
 	${lxcfs} -p ${pidfile} ${LXCFSDIR} &
-	p=$!
+	LXCFSPID=$!
 else
-	pidof lxcfs
-	echo "Using host lxcfs"
+	UNSHARE=0
+	LXCFSPID=$(cat "/run/lxcfs.pid")
+	echo "=> Re-using host lxcfs"
 	rmdir $LXCFSDIR
 	export LXCFSDIR=/var/lib/lxcfs
 fi
 
-trap cleanup EXIT SIGHUP SIGINT SIGTERM
+trap cleanup EXIT HUP INT TERM
 
 count=1
 while ! mountpoint -q $LXCFSDIR; do
@@ -59,9 +69,18 @@ while ! mountpoint -q $LXCFSDIR; do
 done
 
 RUNTEST() {
-	unshare -fmp --mount-proc $*
+	echo ""
+	echo "=> Running ${TESTCASE}"
+
+	if [ "${UNSHARE:-1}" != "0" ]; then
+		unshare -fmp --mount-proc $*
+	else
+		$*
+	fi
 }
 
+TESTCASE="Stress readdir"
+RUNTEST ${dirname}/test_readdir
 TESTCASE="test_proc"
 RUNTEST ${dirname}/test_proc
 TESTCASE="test_cgroup"
@@ -73,7 +92,13 @@ RUNTEST ${dirname}/cpusetrange
 TESTCASE="meminfo hierarchy"
 RUNTEST ${dirname}/test_meminfo_hierarchy.sh
 TESTCASE="liblxcfs reloading"
-${dirname}/test_reload.sh
+UNSHARE=0 RUNTEST ${dirname}/test_reload.sh
+TESTCASE="SIGUSR2 virtualization mode switching"
+echo "==> Switching to non-virtualization mode"
+kill -USR2 $LXCFSPID
+RUNTEST ${dirname}/test_sigusr2.sh
+echo "==> Switching to virtualization mode"
+kill -USR2 $LXCFSPID
 
 # Check for any defunct processes - children we didn't reap
 n=`ps -ef | grep lxcfs | grep defunct | wc -l`
